# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-07 17:18
from __future__ import unicode_literals

from django.db import migrations, models
import image_helper.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=20)),
                ('image', image_helper.fields.SizedImageField(upload_to='sample_images')),
            ],
        ),
    ]
